# 	Copyright (c) 2008 Peter V. Saveliev
#
# 	This file is part of Connexion project.
#
# 	Connexion is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 3 of the License, or
# 	(at your option) any later version.
#
# 	Connexion is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with Connexion; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from command import command, swrapper
from cxutil.exceptions import Dump
from cxutil import flags
from cxutil import cmask
from cxutil import pmask
from xml.dom.minidom import getDOMImplementation


class Default(command):
	'''
	Dump internal tree nodes
	'''
	name = "list"
	mask = flags.Esoteric | flags.Internal | flags.Immediate
	parameters = {
		"prefix": {
			"type": "string",
			"description": "a prefix to list",
			"mask": pmask.Optional
		},
		"namespace": {
			"type": "word",
			"description": "tree namespace",
			"mask": pmask.Optional
		}
	}

	def postUp(self,opts):
		root = opts["esoteric"]["root"]
		oid = opts["oid"]
		creds = opts["creds"]
		prefix = self.fetch(opts.envl,"prefix","")
		namespace = self.fetch(opts.envl,"namespace","")
		bus = opts.esoteric.bus
		xInstance = opts.esoteric.node.parent.module

		if prefix:
			chain = bus.put2("orb",
				{
					"map": "get_chain",
					"creds": creds,
					"text": prefix,
					"namespace": xInstance.namespace,
					"prefix": xInstance.prefix,
				}
			)
			if chain:
				(xInstance,name,rest) = chain[-1]


		imp = getDOMImplementation()
		dom = imp.createDocument(None, "listing", None)
		root = dom.documentElement
		dom.insertBefore(dom.createProcessingInstruction("xml-stylesheet",'href="list.xsl" type="text/xsl"'),root)
		#root.setAttribute("transform","list.xsl")

		namespaces = bus.put2("orb",{ "map": "get_namespaces" } )

		for (i,k,l) in namespaces:		
			if (namespace) and (namespace != i):
				continue
			if i == xInstance.namespace:
				i = xInstance.prefix
			else:
				i = opts.esoteric.node.pwdNS(i)
			r = bus.put2("orb",
				{
					"map": "xmlDump",
					"creds": creds,
					"state": i,
					"prefix": prefix,
					"condition": opts.esoteric.node.parent.condition,
				}
			)
			if r.documentElement.localName == "parameters":
				for i in root.childNodes:
					root.removeNode(i)
				root.appendChild(r.documentElement)
				break

			root.appendChild(r.firstChild)
		
		
		# result = root.xmlDump(creds,"debug",oid)
		s = swrapper()
		s.add_command("data",["text","xml",dom.toxml("utf-8")])
		return s
